@@ -5,7 +5,7 @@ import os
 import re
 from collections import Counter
 from sklearn.model_selection import train_test_split
-from typing import List
+from typing import List, Tuple
 
 from .config import CRFConfig
 from .feature_extraction import (
@@ -201,6 +201,15 @@ class CRFExtractor:
         }
 
 
+    def _remove_duplicates(self, entities: List[Tuple[str, str]]) -> List[Tuple[str, str]]:
+        """
+        Removes duplicates from a list of tuples.
+        """
+        entities_as_str = [json.dumps(e) for e in entities]
+        cleaned_entities = [json.loads(e) for e in list(set(entities_as_str))]
+        return cleaned_entities
+
+
     def _process_tag_output(self, tokens: List[str], sentences: List[List[str]], sent_index: int, field_name: str, label_suffix: str = ""):
         """
         Translates result tokens into entities.
@@ -220,6 +229,8 @@ class CRFExtractor:
                     current_entity = []
         if current_entity:
             entities.append((current_entity_type, " ".join(current_entity)))
+        # Remove duplicates, because all occurrences are going to be detected with pattern.finditer
+        entities = self._remove_duplicates(entities)
         # transform output to facts
         for fact_name, str_val in entities:
             # get spans
