@@ -55,22 +55,18 @@ class FeatureExtractor:
         Checks whether token should be used as a feature.
         Returns boolean.
         """
-
         # TODO: Implement
-
         return False
 
 
-    def _extract_features(self, item: dict, word_index: int, extractors_to_use: list, feature_type: FeatureType):
+    def _extract_features(self, item: dict, word_index: int, extractors_to_use: list, feature_type: FeatureType, token: str):
         """
         Extract features from token and surrounding tokens based on window size.
         """
         features = []
-        
         # select features from selected layers
         if feature_type == FeatureType.MAIN:
             layers_to_use = self.config.feature_layers
-
         elif feature_type == FeatureType.CONTEXT:
             layers_to_use = self.config.context_feature_layers
 
@@ -93,23 +89,21 @@ class FeatureExtractor:
             if layer == FEATURE_LAYER_WORD:
                 for i in range(self.config.suffix_len[0], self.config.suffix_len[1]+1):
                     features.append(f"{word_index}:{layer}.suffix={value[-i:]}")
-        # extract more features from token
-        if FEATURE_LAYER_WORD in item:
-            token = item[FEATURE_LAYER_WORD]
-            for extractor in extractors_to_use:
-                if extractor == EXTRACTOR_ISUPPER:
-                    feature_value = token.isupper()
-                    features.append(f"{word_index}:{FEATURE_LAYER_WORD}.{extractor}={feature_value}")
-                elif extractor == EXTRACTOR_ISTITLE:
-                    feature_value = token.istitle()
-                    features.append(f"{word_index}:{FEATURE_LAYER_WORD}.{extractor}={feature_value}")
-                elif extractor == EXTRACTOR_HASDIGIT:
-                    feature_value = self.has_digit(token)
-                    features.append(f"{word_index}:{FEATURE_LAYER_WORD}.{extractor}={feature_value}")
-                elif extractor == EXTRACTOR_STR_LEN:
-                    feature_value = self.str_length(token)
-                    features.append(f"{word_index}:{FEATURE_LAYER_WORD}.{extractor}={feature_value}")
-                # add more token-based extractors here!
+        # add ectractors
+        for extractor in extractors_to_use:
+            if extractor == EXTRACTOR_ISUPPER:
+                feature_value = token.isupper()
+                features.append(f"{word_index}:{FEATURE_LAYER_WORD}.{extractor}={feature_value}")
+            elif extractor == EXTRACTOR_ISTITLE:
+                feature_value = token.istitle()
+                features.append(f"{word_index}:{FEATURE_LAYER_WORD}.{extractor}={feature_value}")
+            elif extractor == EXTRACTOR_HASDIGIT:
+                feature_value = self.has_digit(token)
+                features.append(f"{word_index}:{FEATURE_LAYER_WORD}.{extractor}={feature_value}")
+            elif extractor == EXTRACTOR_STR_LEN:
+                feature_value = self.str_length(token)
+                features.append(f"{word_index}:{FEATURE_LAYER_WORD}.{extractor}={feature_value}")
+            # add more token-based extractors here!
 
         # get similar lemmas from embedding
         if self.embedding and FEATURE_LAYER_LEMMA in item:
@@ -137,7 +131,8 @@ class FeatureExtractor:
         item = {}
         for layer_name in self.config.feature_layers:
             item[layer_name] = sent[i][0][layer_name]
-        features.extend(self._extract_features(item, "0", self.config.feature_extractors, FeatureType.MAIN))
+        word = sent[i][0][FEATURE_LAYER_WORD]
+        features.extend(self._extract_features(item, "0", self.config.feature_extractors, FeatureType.MAIN, word))
         # check if not the first token in sentence
         if i > 0:
             for window in range(1, self.config.window_size+1):
@@ -145,7 +140,8 @@ class FeatureExtractor:
                     item1 = {}
                     for layer_name in self.config.context_feature_layers:
                         item1[layer_name] = sent[i-window][0][layer_name]
-                    features.extend(self._extract_features(item1, f"-{window}", self.config.context_feature_extractors, FeatureType.CONTEXT))
+                    word1 = sent[i-window][0][FEATURE_LAYER_WORD]
+                    features.extend(self._extract_features(item1, f"-{window}", self.config.context_feature_extractors, FeatureType.CONTEXT, word1))
                 except IndexError:
                     pass
         else:
@@ -158,12 +154,14 @@ class FeatureExtractor:
                     item1 = {}
                     for layer_name in self.config.context_feature_layers:
                         item1[layer_name] = sent[i+window][0][layer_name]
-                    features.extend(self._extract_features(item1, f"+{window}", self.config.context_feature_extractors, FeatureType.CONTEXT))
+                    word1 = sent[i+window][0][FEATURE_LAYER_WORD]
+                    features.extend(self._extract_features(item1, f"+{window}", self.config.context_feature_extractors, FeatureType.CONTEXT, word1))
                 except IndexError:
                     pass
         else:
             # end of sentence
             features.append("EOS")
+        
         return features
 
 
@@ -195,7 +193,9 @@ class FeatureExtractor:
             raise exceptions.InvalidInputError("Document is not a dict!")
         # empty dict for layers
         layers = {}
-        fields_to_check = set(self.config.feature_layers + self.config.context_feature_layers)
+        # always add FEATURE_LAYER_WORD because words might not me used as features,
+        # but we might use the extractive features from words.
+        fields_to_check = set(self.config.feature_layers + self.config.context_feature_layers + [FEATURE_LAYER_WORD])
         # check if mlp fields are present
         for field in fields_to_check:
             if field not in KNOWN_LAYERS:
